@@ -251,18 +251,18 @@ backend_insert_report_data(be, subnet, host, port, script_id, severity, data)
 int _backend_cmpcb(void *unused, harglst *not_used, hargkey_t *lKey,
 	hargtype_t lType, hargkey_t *rKey, hargtype_t rType) 
 {
-	in_addr_t rin,lin;
+	unsigned long rin,lin;
 	unsigned long rip,lip;
-	rin = inet_addr(rKey);
-	lin = inet_addr(lKey);
-	if(rin == -1) {
-		if(lin == -1) {
+	rin = (unsigned long)inet_addr(rKey);
+	lin = (unsigned long)inet_addr(lKey);
+	if(rin == 0xffffffff) {
+		if(lin == 0xffffffff) {
 			return strcmp(rKey,lKey);
 		} else {
 			return 1;
 		}
 	} else {
-		if(lin == -1) {
+		if(lin == 0xffffffff) {
 			return -1;
 		} else {
 			rip = ntohl(rin);
@@ -463,12 +463,12 @@ backend_close(be)
  int be;
 {
  be_info(be, "CLOSE");
+ 
 #ifdef HAVE_MMAP
  if(backends[be].mmap)
  {
   struct stat  buf;
   int len;
-
   fstat(backends[be].fd, &buf);
   len = (int)buf.st_size;
   munmap(backends[be].mmap, len);
@@ -548,6 +548,7 @@ backend_empty(be)
    return -1;
  }
  
+ lseek(backends[be].fd, 0, SEEK_SET);
  f = fdopen(backends[be].fd, "r");
  if ( f == NULL ) 
  {
@@ -560,11 +561,11 @@ backend_empty(be)
   buf[sizeof(buf) - 1] = '\0';
   if(strncmp(buf, "results", strlen("results")) == 0)
    {
-   return 0;
+   return 1;
    }
  }
  
- return 1;
+ return 0;
 }
 
 
