@@ -135,17 +135,11 @@ parse_plugin(buf)
     */
    if((!strcmp(arg_get_value(plugin, "CATEGORY"), "denial")) ||
       (!strcmp(arg_get_value(plugin, "CATEGORY"), "kill_host")) ||
-	   (!strcmp(arg_get_value(plugin, "CATEGORY"), "destructive_attack")))
+	! strcmp(arg_get_value(plugin, "CATEGORY"), "flood") ||
+	   (!strcmp(arg_get_value(plugin, "CATEGORY"), "destructive_attack"))||
+	!strcmp(arg_get_value(plugin, "CATEGORY"), "scanner") )
 	 arg_add_value(plugin, "ENABLED", ARG_INT, sizeof(int), (void*)0);
     else
-	 if(!strcmp(arg_get_value(plugin, "CATEGORY"), "scanner"))
-	 {
-	  if(!strcmp(arg_get_value(plugin, "NAME"), "Nmap"))
-	   arg_add_value(plugin, "ENABLED", ARG_INT, sizeof(int), (void*)1);
-	  else
-	   arg_add_value(plugin, "ENABLED", ARG_INT, sizeof(int), (void*)0);
-	 }
-	 else
 	  arg_add_value(plugin, "ENABLED", ARG_INT, sizeof(int), (void *)1);
 	  
 	  
@@ -225,6 +219,7 @@ comm_plugin_upload(fname)
  char *content;
  char *e;
  int len;
+ 
  e = strrchr(fname, '/');
  if(!e)e = fname;
  else e++;
@@ -238,10 +233,7 @@ comm_plugin_upload(fname)
   return -1;
  }
  
- if( fstat(fd, &stt) < 0 )
- {
-  perror("fstat ");
- }
+ fstat(fd, &stt);
  len = (int)stt.st_size;
  network_printf("CLIENT <|> ATTACHED_PLUGIN\n");
  network_printf("name: %s\n", e);
@@ -452,15 +444,10 @@ comm_get_preferences(prefs)
  Detached_sessions_saved = 0;
 #endif
 
- if(F_quiet_mode)
- {
-  serv_prefs = arg_get_value(prefs, "SERVER_PREFS");
-  if(!serv_prefs)serv_prefs = emalloc(sizeof(struct arglist));
- }
- else
- {
- serv_prefs = emalloc(sizeof(struct arglist));
- }
+ 
+ serv_prefs = arg_get_value(prefs, "SERVER_PREFS");
+ if(!serv_prefs)serv_prefs = emalloc(sizeof(struct arglist));
+ 
  serv_infos = emalloc(sizeof(struct arglist));
  buf = emalloc(32768);
  network_gets(buf, 32768);
@@ -507,13 +494,9 @@ comm_get_preferences(prefs)
 	 }
 	 else
 	  {
-	   if(F_quiet_mode)
-	   {
-	    if(arg_get_type(serv_prefs, pref) < 0)
+	   if(arg_get_type(serv_prefs, pref) < 0)
     	 	arg_add_value(serv_prefs, pref, ARG_STRING, strlen(v), v);
-	   }
-	   else arg_add_value(serv_prefs, pref, ARG_STRING, strlen(v), v);
-	 }
+	  }
        }
     else
     {
@@ -769,7 +752,6 @@ comm_send_preferences(preferences)
  else
   return gui_comm_send_preferences(preferences);
 }
- int len;
 
 int
 comm_send_file(fname)
@@ -779,10 +761,10 @@ comm_send_file(fname)
  struct stat stt;
  long tot = 0;
  char buff[1024];
+ int len;
  
- if(!fname || fname[0] == '\0')
+ if(!fname || !strlen(fname))
   return 0;
- len = (int)stt.st_size;
   
  if(fd < 0)
  {
@@ -791,12 +773,13 @@ comm_send_file(fname)
   return -1;
  }
  
- if ( fstat(fd, &stt) < 0 ) perror("fstat ");
+ fstat(fd, &stt);
+ len = (int)stt.st_size;
  network_printf("CLIENT <|> ATTACHED_FILE\n");
  network_printf("name: %s\n", fname);
  network_printf("content: octet/stream\n");
- tot = (int)stt.st_size;
- network_printf("bytes: %d\n", tot);
+ network_printf("bytes: %d\n", len);
+ tot = len;
  while(tot > 0)
  {
   int m = 0, n;
@@ -918,13 +901,14 @@ int
 comm_get_plugins()
 {
   char * buf;
+  int bufsz;
   int flag = 0;
   int num = 0;
   int num_2 = 0;
   struct arglist * plugin_set;
   struct arglist * scanner_set;
   int ret;
-  
+ 
  /* arg_free_all(Plugins);
   arg_free_all(Scanners);
   */
@@ -933,13 +917,18 @@ comm_get_plugins()
   plugin_set = arg_get_value(Prefs, "PLUGIN_SET");
   scanner_set = arg_get_value(Prefs, "SCANNER_SET");
   
-  buf = emalloc(32768);
+  bufsz = 1024 * 1024;
+  buf = emalloc(bufsz);
   network_gets(buf, 27);
   if(strncmp(buf, "SERVER <|> PLUGIN_LIST <|>", 26))return(-1);
-  bzero(buf, 32768);
   while(!flag)
     {
-      network_gets(buf, 32768);
+      network_gets(buf, bufsz);
+      if ( buf[0] == '\0' ) 
+	{
+	  show_error("The daemon shut down the communication");
+	  break;
+	}
       if(!strncmp(buf, "<|> SERVER", 10))flag = 1;
       else
 	{
@@ -968,6 +957,7 @@ comm_get_plugins()
           else
           {
            num++;
+	
            if(!arg_get_value(Plugins,  arg_get_value(plugin, "NAME")))
 	    arg_add_value(Plugins,  arg_get_value(plugin, "NAME"), ARG_ARGLIST,
 			-1, plugin);
