@@ -72,7 +72,15 @@ be_index_keywords(be)
 {
   int i;
   char * field[BE_NUM_FIELDS];
-  char buf[32768];
+  static char * buf = NULL;
+  static int bufsz = 0;
+
+
+  if ( buf == NULL )
+  {
+   bufsz = 1024 * 1024;
+   buf   = emalloc ( bufsz );
+  }
   
   
   if(backends[be].fields)
@@ -90,12 +98,12 @@ be_index_keywords(be)
   if(eol)
   {
    len = (int)(eol - sol);
-   memcpy(buf, sol, MIN(len, sizeof(buf)));
+   memcpy(buf, sol, MIN(len, bufsz));
   }
   else
   {
    len = strlen(backends[be].lines[i]);
-   memcpy(buf, backends[be].lines[i], MIN(sizeof(buf), len));
+   memcpy(buf, backends[be].lines[i], MIN(bufsz, len));
   }
   
   __split_line(buf, &field[0], &field[1], &field[2], &field[3], &field[4], &field[5], &field[6]);
@@ -215,6 +223,9 @@ mmap_read_line_n(be, buf, size, n)
  int n;
 {
  char* sol, * eol;
+ if ( size <= 0 )
+	return -1;
+ size --;
  sol = backends[be].lines[n];
  if(!sol)
   return -1;
@@ -290,6 +301,9 @@ read_line(be, buf, size)
    }
 	
   eol = backends[be].eols[backends[be].cur_line];
+  if ( size <= 0 )
+	return -1;
+  size --;
   
   if(eol)
   {
@@ -1174,7 +1188,7 @@ execute_query_flatfile(be, query)
  struct query * query;
 {
  struct subset * ret = NULL;
- static char buf[32768];
+ static char buf[1048576];
 #ifdef HAVE_MMAP
  if(backends[be].mmap) backends[be].cur_line = 0;
  else
@@ -1205,7 +1219,7 @@ execute_query_flatfile(be, query)
    { 
    int j;
    char * val = NULL;
-   char buf[32768];
+   char buf[1048576];
    char * table;
    char * subnet;
    char * hostname;
