@@ -481,7 +481,7 @@ void gdImageString(gdImagePtr im, gdFontPtr f,
 {
 	int i;
 	int l;
-	l = strlen(s);
+	l = strlen((char*)s);
 	for (i=0; (i<l); i++) {
 		gdImageChar(im, f, x, y, s[i], color);
 		x += f->w;
@@ -493,7 +493,7 @@ void gdImageStringUp(gdImagePtr im, gdFontPtr f,
 {
 	int i;
 	int l;
-	l = strlen(s);
+	l = strlen((char*)s);
 	for (i=0; (i<l); i++) {
 		gdImageCharUp(im, f, x, y, s[i], color);
 		y -= f->w;
