@@ -8,6 +8,7 @@
 
 #define GDC_INCL
 #define GDC_LIB
+#define GDC_VARS
 #include "gdc.h"
 #include "gdcpie.h"
 
@@ -107,11 +108,11 @@ static int ocmpr( struct tmp_slice_t *a, struct tmp_slice_t *b )
 
 	/* a tie (will happen between each slice) */
 	/* are we within pie_3D_rad */
-	if( (a->angle < pie_3D_rad) && (pie_3D_rad < a->slice) ||
-		(a->slice < pie_3D_rad) && (pie_3D_rad < a->angle) )
+	if( ((a->angle < pie_3D_rad) && (pie_3D_rad < a->slice)) ||
+		((a->slice < pie_3D_rad) && (pie_3D_rad < a->angle)) )
 		return 1;
-	if( (b->slice < pie_3D_rad) && (pie_3D_rad < b->angle) ||
-		(b->angle < pie_3D_rad) && (pie_3D_rad < b->slice) )
+	if( ((b->slice < pie_3D_rad) && (pie_3D_rad < b->angle)) ||
+		((b->angle < pie_3D_rad) && (pie_3D_rad < b->slice)) )
 		return -1;
 
 	/* let slice angle decide */
@@ -146,8 +147,8 @@ pie_gif( short			GIFWIDTH,
 	int			BGColor,
 				LineColor,
 				PlotColor,
-				EdgeColor,
-				EdgeColorShd,
+				EdgeColor = 0,
+				EdgeColorShd = 0,
 #ifdef USE_ALLOCA
 				*SliceColor = (int*)alloca (num_points * sizeof (int)),
 				*SliceColorShd = (int*)alloca (num_points * sizeof (int));
@@ -165,9 +166,6 @@ pie_gif( short			GIFWIDTH,
 	float		min_grphable = ( GDCPIE_other_threshold < 0?
 								  100.0/(float)MIN(GIFWIDTH,GIFHEIGHT):
 								  (float)GDCPIE_other_threshold )/100.0;
-	short		num_slices1 = 0,
-				num_slices2 = 0;
-	char		any_too_small = FALSE;
 #ifdef USE_ALLOCA
 	char		*others = (char*)alloca (num_points);
 	float		*slice_angle = (float*)alloca (3 * num_points * sizeof (float));
@@ -179,8 +177,7 @@ pie_gif( short			GIFWIDTH,
 
 	int			xdepth_3D      = 0,			/* affects PX()*/
 				ydepth_3D      = 0;			/* affects PY()*/
-	int			do3Dx = 0,					/* reserved for macro use*/
-				do3Dy = 0;
+			 						/* reserved for macro use*/
 
 /*	GDCPIE_3d_angle = MOD_360(90-GDCPIE_3d_angle+360);*/
 	pie_3D_rad = TO_RAD( GDCPIE_3d_angle );
@@ -206,7 +203,6 @@ pie_gif( short			GIFWIDTH,
 									   + 2:
 									   0;
 	float	last = 0.0;
-	float	label_explode_limit = 0.0;
 	int		cheight,
 			cwidth;
 
@@ -386,7 +382,7 @@ pie_gif( short			GIFWIDTH,
 	/*              determined by number of labels that'll fit (height) */
 	/* scale to user values */
 	/* ( M_PI / (GIFHEIGHT / (SFONTHGT+1)) ) */
-/*	min_grphable = tot_val /*/
+/*	min_grphable = tot_val */
 /*				   ( 2.0 * (float)GIFHEIGHT / (float)(SFONTHGT+1+TFONTHGT+2) );*/
 
 
@@ -581,7 +577,7 @@ pie_gif( short			GIFWIDTH,
 				int		lbl_wdth;
 				short	num_nl = cnt_nl( lbl[i], &lbl_wdth );
 				int		lblx,  pctx,
-						lbly,  pcty,
+						lbly,  pcty = 0,
 						linex, liney;
 
 				lbl_wdth *= GDC_fontc[GDCPIE_label_size].w;
@@ -665,7 +661,7 @@ pie_gif( short			GIFWIDTH,
 	return;
 }
 
-/* $Id: gdc_pie.c,v 1.6 2000/10/29 00:21:59 renaud Exp $
+/* $Id: gdc_pie.c,v 1.7 2003/05/21 16:05:37 renaud Exp $
  * local variables:
  * mode: c
  * tab-width: 4
