@@ -144,6 +144,13 @@ monitor_dialog_setup(victim, restore)
   int tag;
   int backend = backend_init(NULL);
 
+  /* Could not create a backend */
+  if( backend < 0 )
+  {
+    gtk_widget_show(arg_get_value(MainDialog, "WINDOW"));
+    return;
+  }
+  
 
   arg_add_value(ctrls, "MONITOR_BACKEND", ARG_INT, -1, (void*)backend);
   
@@ -507,7 +514,7 @@ monitor_stop_whole_test(w, ctrls)
   */
  while(type != MSG_BYE)
  {
-  bzero(buf, sizeof(buf));
+  buf[sizeof(buf) - 1] = '\0';
   network_gets_raw(buf, sizeof(buf) - 1);
    if( buf[0] == '\0') {
   	break;
@@ -538,11 +545,19 @@ monitor_input_callback(ctrls, fd, condition)
     int condition;
 {
   int finished = 0;
-  char buf [32678];
-  char * msg;
+  static char * buf = NULL;
+  static int    bufsz = 0;
+  static char * msg  = NULL;
   int n, type = -1;
   int interrupted = 0;
-  network_gets_raw(buf, 32678);
+
+  if ( buf == NULL )
+  {
+   bufsz = 1024 * 1024;
+   buf   = emalloc( bufsz );
+   msg   = emalloc( bufsz );
+  }
+  network_gets_raw( buf, bufsz );
   if ((n = strlen (buf)) && buf [n-1] == '\n') buf [n-1] = '\0';
 
   if( buf[0] == '\0') {
@@ -553,7 +568,7 @@ monitor_input_callback(ctrls, fd, condition)
 	}
         else return;
        }	
-  msg  = emalloc(strlen(buf)+1);
+
   type = parse_server_message(buf, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), msg);
  
   switch(type)
@@ -575,8 +590,8 @@ monitor_input_callback(ctrls, fd, condition)
 		monitor_remove_host(ctrls, msg);
 		break;	
   }
-  bzero(buf, sizeof(buf));
-  efree(&msg);
+  buf[0] = '\0';
+  msg[0] = '\0';
   
   if(finished)
     {
