@@ -15,6 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  */
 
 #include <includes.h>
@@ -36,7 +46,7 @@ static void detached_cb(w, ctrls)
  struct arglist * ctrls;
 {
  GtkWidget * cont;
- GtkWidget * label, * entry;
+ GtkWidget * label, * entry, *email, *email_label;
  
  if(GTK_TOGGLE_BUTTON(w) -> active)
  {
@@ -49,9 +59,14 @@ See http://www.nessus.org/doc/detached_scan.html for details\n");
  cont  = arg_get_value(ctrls, "CONTINUOUS_SCAN");
  label = arg_get_value(ctrls, "DELAY_LABEL");
  entry = arg_get_value(ctrls, "DELAY");
-  
-  
+ email = arg_get_value(ctrls, "EMAIL_ADDR"); 
+ email_label = arg_get_value(ctrls, "EMAIL_ADDR_LABEL");
+ 
  gtk_widget_set_sensitive(cont, GTK_TOGGLE_BUTTON(w) -> active);
+ gtk_widget_set_sensitive(email, GTK_TOGGLE_BUTTON(w)->active);
+ gtk_widget_set_sensitive(email_label, GTK_TOGGLE_BUTTON(w)->active);
+   
+   
  if(!GTK_TOGGLE_BUTTON(w) -> active)
  {
   gtk_widget_set_sensitive(label, FALSE);
@@ -62,6 +77,8 @@ See http://www.nessus.org/doc/detached_scan.html for details\n");
    gtk_widget_set_sensitive(label, GTK_TOGGLE_BUTTON(cont)->active);
    gtk_widget_set_sensitive(entry, GTK_TOGGLE_BUTTON(cont)->active);
  }
+ 
+ 
 }
 static void continuous_cb(w, ctrls)
  GtkWidget* w;
@@ -71,9 +88,8 @@ static void continuous_cb(w, ctrls)
   if(GTK_TOGGLE_BUTTON(w) -> active)
  {
   show_warning("If you enable this option, you are asking nessusd to\n\
-continuously scan your network, again and again. As there is no way\n\
-to remotely stop this mode, be sure you have read\n\
-http://www.nessus.org/doc/detached_scan.html first !\n");
+continuously scan your network, again and again.\n\
+Be sure to have read http://www.nessus.org/doc/detached_scan.html first !\n");
  }
  label = arg_get_value(ctrls, "DELAY_LABEL");
  entry = arg_get_value(ctrls, "DELAY");
@@ -88,10 +104,13 @@ struct arglist * prefs_dialog_scan_opt()
  GtkWidget * table;
  GtkWidget * ping_hosts;
  GtkWidget * optimize_test;
+ GtkWidget * safe_checks;
+ GtkWidget * use_mac_addr;
  GtkWidget * reverse_lookup;
  GtkWidget * box;
  GtkWidget * label;
  GtkWidget * port_range;
+ GtkWidget * unscanned_as_closed;
  GtkWidget * entry;
  GtkWidget * scanners_window;
  GtkWidget * list;
@@ -123,7 +142,7 @@ struct arglist * prefs_dialog_scan_opt()
  
 
  
- table = gtk_table_new(3, 2, FALSE);
+ table = gtk_table_new(5, 2, FALSE);
  gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
  gtk_widget_show(table);
  
@@ -138,27 +157,48 @@ struct arglist * prefs_dialog_scan_opt()
  gtk_widget_show(port_range);
  arg_add_value(ctrls, "PORT_RANGE", ARG_PTR, -1, port_range);
  
+ unscanned_as_closed = gtk_check_button_new_with_label("Consider unscanned ports as closed");
+ arg_add_value(ctrls, "UNSCANNED_CLOSED", ARG_PTR, -1, unscanned_as_closed);
+ gtk_table_attach_defaults(GTK_TABLE(table), unscanned_as_closed, 0,2,1,2);
+ gtk_widget_show(unscanned_as_closed);
+ 
+ 
+ 
+ 
+ 
  gtk_table_set_row_spacing(GTK_TABLE(table), 1, 10);
- label = gtk_label_new("Max threads : ");
- gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,1,2);
- gtk_widget_show(label);
- 
- entry = gtk_entry_new();
- gtk_table_attach_defaults(GTK_TABLE(table), entry, 1,2,1,2);
- gtk_widget_show(entry);
- arg_add_value(ctrls, "MAX_THREADS", ARG_PTR, -1, entry);
- 
- label = gtk_label_new("Path to the CGIs : ");
+ label = gtk_label_new("Number of hosts to test at the same time : ");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,2,3);
  gtk_widget_show(label);
  
  entry = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1,2,2,3);
  gtk_widget_show(entry);
+ arg_add_value(ctrls, "MAX_HOSTS", ARG_PTR, -1, entry);
+ 
+ gtk_table_set_row_spacing(GTK_TABLE(table), 2, 10);
+ label = gtk_label_new("Number of checks to perform at the same time : ");
+ gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,3,4);
+ gtk_widget_show(label);
+ 
+ entry = gtk_entry_new();
+ gtk_table_attach_defaults(GTK_TABLE(table), entry, 1,2,3,4);
+ gtk_widget_show(entry);
+ arg_add_value(ctrls, "MAX_CHECKS", ARG_PTR, -1, entry);
+ 
+ 
+
+ label = gtk_label_new("Path to the CGIs : ");
+ gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,4,5);
+ gtk_widget_show(label);
+ 
+ entry = gtk_entry_new();
+ gtk_table_attach_defaults(GTK_TABLE(table), entry, 1,2,4,5);
+ gtk_widget_show(entry);
  arg_add_value(ctrls, "CGI_PATH", ARG_PTR, -1, entry);
  
  
-  reverse_lookup = gtk_check_button_new_with_label("Do a reverse lookup on the IP before testing it");
+ reverse_lookup = gtk_check_button_new_with_label("Do a reverse lookup on the IP before testing it");
  arg_add_value(ctrls, "REVERSE_LOOKUP", ARG_PTR, -1, reverse_lookup);
  gtk_box_pack_start(GTK_BOX(box), reverse_lookup, FALSE, FALSE, 0);
  gtk_widget_show(reverse_lookup);
@@ -168,7 +208,20 @@ struct arglist * prefs_dialog_scan_opt()
  gtk_box_pack_start(GTK_BOX(box), optimize_test, FALSE, FALSE, 0);
  gtk_widget_show(optimize_test);
  
+ 
+ safe_checks = gtk_check_button_new_with_label("Safe checks");
+ arg_add_value(ctrls, "SAFE_CHECKS", ARG_PTR, -1, safe_checks);
+ gtk_box_pack_start(GTK_BOX(box), safe_checks, FALSE, FALSE, 0);
+ gtk_widget_show(safe_checks);
+ 
+ use_mac_addr = gtk_check_button_new_with_label("Designate hosts by their MAC address");
+ arg_add_value(ctrls, "USE_MAC_ADDR", ARG_PTR, -1, use_mac_addr);
+ gtk_box_pack_start(GTK_BOX(box), use_mac_addr, FALSE, FALSE, 0);
+ gtk_widget_show(use_mac_addr);
+ 
+
 #ifdef ENABLE_SAVE_KB
+#if 0
  opt = gtk_check_button_new_with_label("Detached scan");
  arg_add_value(ctrls, "DETACHED_SCAN", ARG_PTR, -1, opt);
  gtk_box_pack_start(GTK_BOX(box), opt, FALSE, FALSE, 0);
@@ -178,6 +231,23 @@ struct arglist * prefs_dialog_scan_opt()
 		     GTK_SIGNAL_FUNC(detached_cb),
 		     ctrls);
  
+ hbox = gtk_hbox_new(TRUE, TRUE);
+ /*gtk_container_set_border_width(GTK_CONTAINER(hbox), 5); */
+ gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, TRUE, 0);
+ gtk_widget_show(hbox);
+ label = gtk_label_new("Send results to this email address : ");
+ arg_add_value(ctrls, "EMAIL_ADDR_LABEL", ARG_PTR, -1, label);
+ gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
+ gtk_widget_set_sensitive(label, FALSE);
+ gtk_widget_show(label);
+ 
+ entry = gtk_entry_new();
+ arg_add_value(ctrls, "EMAIL_ADDR", ARG_PTR, -1, entry);
+ gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
+ gtk_widget_set_sensitive(entry, FALSE);
+ gtk_widget_show(entry);
+ 
+ 
  opt = gtk_check_button_new_with_label("Continuous scan");
  arg_add_value(ctrls, "CONTINUOUS_SCAN", ARG_PTR, -1, opt);
  gtk_box_pack_start(GTK_BOX(box), opt, FALSE, FALSE, 0);
@@ -206,6 +276,7 @@ struct arglist * prefs_dialog_scan_opt()
  gtk_box_pack_start(GTK_BOX(hbox), opt, TRUE, TRUE, 0);
  gtk_widget_set_sensitive(opt, FALSE);
  gtk_widget_show(opt);
+#endif
 #endif
  scanners_window = gtk_scrolled_window_new(NULL,NULL);
  gtk_container_border_width(GTK_CONTAINER(scanners_window), 0);
@@ -305,7 +376,7 @@ fill_scanner_list(ctrls)
    gtk_list_append_items(GTK_LIST(arg_get_value(ctrls, "SCANNERS_LIST")), dlist);
 }
 
-void prefs_scanner_redraw(w, dumb, ctrls)
+int prefs_scanner_redraw(w, dumb, ctrls)
  GtkWidget * w;
  void * dumb;
  struct arglist * ctrls;
@@ -317,6 +388,7 @@ void prefs_scanner_redraw(w, dumb, ctrls)
   fill_scanner_list(ctrls);
   arg_set_value(ctrls, "SCANNERS_NUM", sizeof(int), (void *)ScannersNum);
   }
+ return 0;
 }
 
 
