@@ -710,8 +710,8 @@ int backend_to_xml_ng(be, filename)
  xml_fprintf (fd, 0, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n");
  xml_fprintf (fd, 0, "<!-- nessus.xsl can be found in the nessus-tools directory -->\n");
 
- xml_fprintf (fd, 0, "<?xml:stylesheet type=\"text/xsl\" href=\"nessus.xsl\"?>\n");
- xml_fprintf (fd, 0, "<report version=\"1.3\">\n");
+ xml_fprintf (fd, 0, "<?xml-stylesheet type=\"text/xsl\" href=\"nessus.xsl\"?>\n");
+ xml_fprintf (fd, 0, "<report version=\"1.4\">\n");
  /* don't need this: fprintf (fd, "\t<scan>\n"); */
   xml_info (be, fd, 1); 
   xml_config (fd, 1); 
