@@ -19,8 +19,7 @@
 #ifndef __NESSUS_ATTACK_H__
 #define __NESSUS_ATTACK_H__
 
-struct arglist * attack_init_hostinfos(char *, struct in_addr *);
-int attack_add_hostinfos(struct arglist *, struct arglist *);
-int attack_host(struct arglist *);
-void attack_start(struct arglist *);
+
+int attack_network(struct arglist *);
+
 #endif
