@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1998 - 2003 Renaud Deraison
+ * Copyright (C) 1998 - 2004 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2,
@@ -14,16 +14,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * In addition, as a special exception, Renaud Deraison
- * gives permission to link the code of this program with any
- * version of the OpenSSL library which is distributed under a
- * license identical to that listed in the included COPYING.OpenSSL
- * file, and distribute linked combinations including the two.
- * You must obey the GNU General Public License in all respects
- * for all of the code used other than OpenSSL.  If you modify
- * this file, you may extend this exception to your version of the
- * file, but you are not obligated to do so.  If you do not wish to
- * do so, delete this exception statement from your version.
  *
  * pluginscheduler.c : 
  *  Tells nessusd which plugin should be executed now
@@ -35,42 +25,41 @@
 #include "pluginscheduler.h"
 #include "pluginload.h"
 #include "pluginlaunch.h"
+#include "preferences.h"
+#include "log.h"
 
 
-#define HASH_MAX 2713
+#define HASH_MAX 9973
 
 
+/*-----------------------------------------------------------------------------*/
 
-int plugin_get_running_state(struct arglist * plugin)
+int plugin_get_running_state(struct scheduler_plugin * plugin)
 {
- return (int)arg_get_value(plugin, "RUNNING_STATE");
+ return plugin->running_state;
 }
 
-void plugin_set_running_state(struct arglist * plugin, int state)
+void plugin_set_running_state(plugins_scheduler_t shed, struct scheduler_plugin * plugin, int state)
 {
  if(plugin == NULL)
   return;
-  
- if(arg_get_value(plugin, "RUNNING_STATE") != NULL)
-  arg_set_value(plugin, "RUNNING_STATE", sizeof(state), (void*)state);
- else 
-  arg_add_value(plugin, "RUNNING_STATE", ARG_INT, sizeof(state), (void*)state);
+
+ plugin->running_state = state;
 }
 
 /*-----------------------------------------------------------------------------*/
 
 
-static int mkhash(char * name)
+static unsigned int mkhash(char * name)
 {
-  int l = strlen(name);
   int i;
   int h = 0;
   
-  for(i=0;i<l;i++)
+  for(i=0; name[i] != '\0' ;i++)
    h = ((h * 128) + name[i]) % HASH_MAX;
    
  
-  return h < 0 ? -h:h;
+  return h;
 }
 
 /*------------------------------------------------------------------------------*/
@@ -97,7 +86,7 @@ cache_get_name(name)
  char * name;
 {
  struct name_cache * nc;
- int h;
+ unsigned int h;
  
  if(cache_inited == 0)cache_init();
  
@@ -123,7 +112,7 @@ cache_add_name(name)
  char * name;
 {
  struct name_cache * nc;
- int h;
+ unsigned int h;
  
  if(name == NULL)
   return NULL;
@@ -137,6 +126,9 @@ cache_add_name(name)
  nc->occurences = 1;
  nc->next = cache[h].next;
  nc->prev = NULL;
+ if ( cache[h].next != NULL )
+  cache[h].next->prev = nc;
+
  cache[h].next = nc;
  
  return nc;
@@ -172,7 +164,7 @@ cache_dec(name)
  nc->occurences --;
  if( nc->occurences == 0 )
   {
-    int h = mkhash(name);
+    unsigned int h = mkhash(name);
     efree(&nc->name);
 
     if( nc->next != NULL)
@@ -182,10 +174,13 @@ cache_dec(name)
      nc->prev->next = nc->next;
     else
      cache[h].next = nc->next;
+
+    efree(&nc);
   }
 }
 
 
+
 /*---------------------------------------------------------------------------*
  *
  * A minimalist HASH stucture
@@ -218,6 +213,12 @@ static void hash_link_destroy(struct hash * h)
   }
   efree(&h->dependencies);
  }
+
+ arg_free_all(h->plugin->required_ports);
+ arg_free_all(h->plugin->required_udp_ports);
+ arg_free_all(h->plugin->required_keys);
+ arg_free_all(h->plugin->excluded_keys);
+ efree(&h->plugin);
  
  if( h->ports != NULL )
  {
@@ -243,12 +244,12 @@ static void hash_destroy(struct hash * h)
 }
 
 
-static int hash_add(struct hash * h, char * name, struct arglist * plugin)
+static int hash_add(struct hash * h, char * name, struct scheduler_plugin * plugin)
 {
  struct hash * l = emalloc(sizeof(struct hash));
- int idx = mkhash(name);
- struct arglist * deps = plug_get_deps(plugin->value);
- struct arglist * ports = plug_get_required_ports(plugin->value);
+ unsigned int idx = mkhash(name);
+ struct arglist * deps = plug_get_deps(plugin->arglist->value);
+ struct arglist * ports = plug_get_required_ports(plugin->arglist->value);
  int num_deps = 0;
  
  l->plugin = plugin;
@@ -307,7 +308,7 @@ static int hash_add(struct hash * h, char * name, struct arglist * plugin)
 
 static struct hash * _hash_get(struct hash * h, char * name)
 {
- int idx = mkhash(name);
+ unsigned int idx = mkhash(name);
  struct hash * l = h[idx].next;
  while(l != NULL)
  {
@@ -319,7 +320,7 @@ static struct hash * _hash_get(struct hash * h, char * name)
  return NULL;
 }
 
-static struct arglist * hash_get(struct hash * h, char * name)
+static struct scheduler_plugin * hash_get(struct hash * h, char * name)
 {
  struct hash * l = _hash_get(h, name);
  if(  l == NULL )
@@ -341,8 +342,6 @@ static char ** hash_get_ports(struct hash * h, char * name)
 static char ** hash_get_deps(struct hash * h, char * name)
 {
  struct hash * l = _hash_get(h, name);
- char ** deps;
- int i;
  
  if( l == NULL )
   return NULL;
@@ -350,35 +349,7 @@ static char ** hash_get_deps(struct hash * h, char * name)
  if( l->dependencies == NULL )
   return NULL;
  
- /*
-  * Check if any of the dependencies have run already, and remove them
-  */
- i = 0;
- deps = l->dependencies;
- 
-#if 0
- while ( deps[i] != NULL )
- {
-  p = hash_get(h, deps[i]);
-  if(p == NULL)
-  	fprintf(stderr, "hash_get_deps(): Error - can't find plugin\n");
-  else 
-  	{
-	 int state = plugin_get_running_state(p->value);
-	 if(state == PLUGIN_STATUS_DONE || state ==  PLUGIN_STATUS_DONE_AND_CLEANED)
-	 {
-	  l->num_deps --;
-	  cache_dec(deps[i]);
-	  deps[i] = NULL;
-	  if(l->num_deps > 0)
-	  	memmove(&(deps[i]), &(deps[i+1]), ( l->num_deps - i ) * sizeof(char*));
-	  i--;
-	 }
-	}
-  i ++;	
- }
-#endif 
- return deps;
+ return l->dependencies;
 }
 
 
@@ -405,9 +376,9 @@ struct plist * pl_get(struct plist * list, char * name)
 
 
 
-void scheduler_mark_running_ports(plugins_scheduler_t sched, struct arglist * plugin)
+void scheduler_mark_running_ports(plugins_scheduler_t sched, struct scheduler_plugin * plugin)
 {
- char ** ports = hash_get_ports(sched->hash, plugin->name);
+ char ** ports = hash_get_ports(sched->hash, plugin->arglist->name);
  int i;
  
  if( ports == NULL )
@@ -433,12 +404,14 @@ void scheduler_mark_running_ports(plugins_scheduler_t sched, struct arglist * pl
  } 
 }
 
-void scheduler_rm_running_ports(plugins_scheduler_t sched, struct arglist * plugin)
+void scheduler_rm_running_ports(plugins_scheduler_t sched, struct scheduler_plugin * plugin)
 {
- char ** ports = hash_get_ports(sched->hash, plugin->name);
+ char ** ports; 
  int i;
 
+
  
+ ports = hash_get_ports(sched->hash, plugin->arglist->name);
 
  
  if( ports == NULL )
@@ -470,14 +443,16 @@ void scheduler_rm_running_ports(plugins_scheduler_t sched, struct arglist * plug
 }
 
 
+#if DISABLED_AND_BROKEN
+
 /*
  * Returns the 'score' of the plugin, which means the number of
  * plugins that are already hammering the port this plugin will
  * hammer too
  */
-int scheduler_plugin_score(plugins_scheduler_t sched, struct arglist * plugin)
+int scheduler_plugin_score(plugins_scheduler_t sched, struct scheduler_plugin * plugin)
 {
- char ** ports = hash_get_ports(sched->hash, plugin->name);
+ char ** ports = hash_get_ports(sched->hash, plugin->arglist->name);
  int i;
  int score = 0;
  
@@ -510,11 +485,12 @@ void scheduler_plugin_best_score(plugins_scheduler_t sched, int *bscore, struct
 }
 
 
+#endif
 
 
 
 
-struct arglist * plugin_next_unrun_dependencie(plugins_scheduler_t sched, char ** dependencies)
+struct scheduler_plugin * plugin_next_unrun_dependencie(plugins_scheduler_t sched, char ** dependencies, int already_in_dependencie)
 {
  struct hash * h = sched->hash;
  int flag = 0;
@@ -526,22 +502,22 @@ struct arglist * plugin_next_unrun_dependencie(plugins_scheduler_t sched, char *
   
  for(i=0;dependencies[i] != NULL;i++)
   {
-   struct arglist * plugin = hash_get(h, dependencies[i]);
+   struct scheduler_plugin * plugin = hash_get(h, dependencies[i]);
    if(plugin != NULL)
    {
-    int state = plugin_get_running_state(plugin->value);
+    int state = plugin_get_running_state(plugin);
     switch(state)
     {
      case PLUGIN_STATUS_UNRUN :
      	{
-	char ** deps = hash_get_deps(h, plugin->name);
-	struct arglist * ret;
+	char ** deps = hash_get_deps(h, plugin->arglist->name);
+	struct scheduler_plugin * ret;
 	counter ++;
 	if(deps == NULL)
 	  return plugin;	
 	else
 	 {
-	 ret = plugin_next_unrun_dependencie(sched, deps);
+	 ret = plugin_next_unrun_dependencie(sched, deps, 1);
 	 if(ret == NULL)
 		return plugin;
 	 else 
@@ -554,8 +530,8 @@ struct arglist * plugin_next_unrun_dependencie(plugins_scheduler_t sched, char *
      	flag++;
 	break;
      case PLUGIN_STATUS_DONE:
-     	scheduler_rm_running_ports(sched, plugin);
-	plugin_set_running_state(plugin->value, PLUGIN_STATUS_DONE_AND_CLEANED);
+ 	scheduler_rm_running_ports(sched, plugin);	
+	plugin_set_running_state(sched, plugin, PLUGIN_STATUS_DONE_AND_CLEANED);
 	break;
      case PLUGIN_STATUS_DONE_AND_CLEANED:
      	break;
@@ -576,33 +552,40 @@ struct arglist * plugin_next_unrun_dependencie(plugins_scheduler_t sched, char *
 /*
  * Enables a plugin and its dependencies
  */
-static void enable_plugin_and_dependencies(plugins_scheduler_t shed, struct arglist * plugin, char * name)
+static void enable_plugin_and_dependencies(plugins_scheduler_t shed, struct arglist * plugin, char * name, int silent)
 {
  char ** deps;
  int i;
+ int status;
  
  deps = hash_get_deps(shed->hash, name);
 
- plug_set_launch(plugin, 1);
+ status = plug_get_launch(plugin);
+ if ( status == LAUNCH_DISABLED )
+ {
+  if ( silent == 0 )
+   plug_set_launch(plugin, LAUNCH_RUN);
+  else
+   plug_set_launch(plugin, LAUNCH_SILENT);
+ }
  
  if(deps != NULL)
  {
    for(i=0;deps[i] != NULL;i++)
     {
-     struct arglist * p;
+     struct scheduler_plugin * p;
      p = hash_get(shed->hash, deps[i]);
      if( p != NULL )
-       enable_plugin_and_dependencies(shed, p->value, p->name);
+       enable_plugin_and_dependencies(shed, p->arglist->value, p->arglist->name, silent);
      else
-       fprintf(stderr, "'%s' depends on '%s' which could not be found\n", name, deps[i]);
-     
+       log_write("'%s' depends on '%s' which could not be found\n", name, deps[i]);
     }
  }
 }
 
 /*---------------------------------------------------------------------------*/
 
-plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoload)
+plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoload, int silent_dependencies)
 {
  plugins_scheduler_t ret = emalloc(sizeof(*ret));
  struct arglist * arg;
@@ -622,21 +605,33 @@ plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoloa
   arg = plugins;
   while(arg->next != NULL)
   {
+  struct scheduler_plugin * scheduler_plugin;
   struct list * dup;
-  struct arglist * args = arg->value;
-  int category =  plug_get_category(args);
+  int category =  plug_get_category(arg->value);
   
+  scheduler_plugin = emalloc ( sizeof(struct scheduler_plugin) ) ;
+  scheduler_plugin->arglist  = arg;
+  scheduler_plugin->running_state  = PLUGIN_STATUS_UNRUN;
+  scheduler_plugin->category = plug_get_category(arg->value);
+  scheduler_plugin->timeout  = plug_get_timeout(arg->value);
+
+  scheduler_plugin->required_ports = plug_get_required_ports(arg->value);
+  scheduler_plugin->required_udp_ports = plug_get_required_udp_ports(arg->value);
+  scheduler_plugin->required_keys = plug_get_required_keys(arg->value);
+  scheduler_plugin->excluded_keys = plug_get_excluded_keys(arg->value);
+ 
+
   if(category > ACT_LAST)category = ACT_LAST;
   dup = emalloc(sizeof(struct list));
-  dup->name = arg->name;
-  dup->plugin = arg;
+  dup->name = scheduler_plugin->arglist->name;
+  dup->plugin = scheduler_plugin;
   dup->prev = NULL;
   dup->next = ret->list[category];
   if(ret->list[category] != NULL)
    ret->list[category]->prev = dup;
   ret->list[category] = dup;
-  hash_add(ret->hash, arg->name, arg);
-  plugin_set_running_state(arg->value, PLUGIN_STATUS_UNRUN);
+
+  hash_add(ret->hash, arg->name, scheduler_plugin);
   arg = arg->next;
   }
  
@@ -646,8 +641,8 @@ plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoloa
  arg = plugins;
  while(arg->next != NULL)
   {
-   if(plug_get_launch(arg->value) != 0)
-   	enable_plugin_and_dependencies(ret, arg->value, arg->name);
+   if(plug_get_launch(arg->value) != LAUNCH_DISABLED)
+   	enable_plugin_and_dependencies(ret, arg->value, arg->name, silent_dependencies);
    arg = arg->next;
   }
  }
@@ -659,7 +654,7 @@ plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoloa
   struct list * l = ret->list[i];
   while (l != NULL )
   {
-   if(plug_get_launch(l->plugin->value) == 0)
+   if(plug_get_launch(l->plugin->arglist->value) == LAUNCH_DISABLED)
    {
     struct list * old = l->next;
 
@@ -669,11 +664,11 @@ plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoloa
       ret->list[i] = l->next;
 	  
 	 
-	 if(l->next != NULL)
+     if(l->next != NULL)
 	  l->next->prev = l->prev;
 
-	efree(&l);
-	l = old;
+        efree(&l);
+        l = old;
 	continue;
     }
     l = l->next;
@@ -685,7 +680,7 @@ plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoloa
 
 
 
-struct arglist * plugins_scheduler_next(plugins_scheduler_t h)
+struct scheduler_plugin * plugins_scheduler_next(plugins_scheduler_t h)
 {
  
  struct list * l;
@@ -706,6 +701,7 @@ struct arglist * plugins_scheduler_next(plugins_scheduler_t h)
 
  if((category == ACT_SCANNER) ||
     (category == ACT_KILL_HOST) ||
+    (category == ACT_FLOOD) ||
     (category == ACT_DENIAL))
     pluginlaunch_disable_parrallel_checks();
  else
@@ -716,58 +712,40 @@ struct arglist * plugins_scheduler_next(plugins_scheduler_t h)
  {
    int state;
   
-  state = plugin_get_running_state(l->plugin->value);
+  state = plugin_get_running_state(l->plugin);
  
   
   switch(state)
   {
    case PLUGIN_STATUS_UNRUN:
     {
-    char ** deps = hash_get_deps(h->hash, l->plugin->name);
+    char ** deps = hash_get_deps(h->hash, l->plugin->arglist->name);
     
     if(deps != NULL)
     {
-     struct arglist * p = plugin_next_unrun_dependencie(h, deps);
+     struct scheduler_plugin * p = plugin_next_unrun_dependencie(h, deps, 0);
      
      switch((int)p)
      {
       case (int)NULL :
       	scheduler_mark_running_ports(h, l->plugin);
-        plugin_set_running_state(l->plugin->value, PLUGIN_STATUS_RUNNING);
+        plugin_set_running_state(h, l->plugin, PLUGIN_STATUS_RUNNING);
         return l->plugin;
 
 	 break;
      case (int)PLUG_RUNNING:
         {
-	int cat;
-     		/* One of the dependencie is still running */
-	cat = plug_get_category(l->plugin->value);
-	if(cat < running_category)
-		running_category = cat;
+     	/* One of the dependencie is still running  -  we write down its category */
+	if(l->plugin->category < running_category)
+		running_category = l->plugin->category;
      	flag ++;
 	}
 	break;
      default:
         { 
-	 /* Launch a dependencie */
-	
-	 
-	 /* Ack - the plugin we depend on is not of the same category - we recast the current
-	  * plugin to another (greater) category 
-	  */
-#if 0	  
-		   int c;
-
-	 if((c = plug_get_category(p->value)) != category)
-	 {
-	  struct arglist * args = l->plugin->value;
-	  arg_set_value(args, "CATEGORY", sizeof(int), (void*)c); /* XXXXXXX */
-	  l = l->next;
-	  continue;
-	 }
-#endif	 
+	 /* Launch a dependencie  - don't pay attention to the type */
 	scheduler_mark_running_ports(h, p);
-        plugin_set_running_state(p->value, PLUGIN_STATUS_RUNNING);
+        plugin_set_running_state(h, p, PLUGIN_STATUS_RUNNING);
 	return p;
        }
     }
@@ -775,23 +753,22 @@ struct arglist * plugins_scheduler_next(plugins_scheduler_t h)
     else /* No dependencies */
      {
         scheduler_mark_running_ports(h, l->plugin);
-        plugin_set_running_state(l->plugin->value, PLUGIN_STATUS_RUNNING);
+        plugin_set_running_state(h, l->plugin, PLUGIN_STATUS_RUNNING);
         return l->plugin;
      }
    }
     break;
   case PLUGIN_STATUS_RUNNING:
         {
-	int cat = plug_get_category(l->plugin->value);
-	if(cat < running_category)
-		running_category = cat;
+	if(l->plugin->category < running_category)
+		running_category = l->plugin->category;
   	flag ++;
 	}
 	break;
 
   case PLUGIN_STATUS_DONE :
   	scheduler_rm_running_ports(h, l->plugin);
-	plugin_set_running_state(l->plugin->value, PLUGIN_STATUS_DONE_AND_CLEANED);
+	plugin_set_running_state(h, l->plugin, PLUGIN_STATUS_DONE_AND_CLEANED);
 	/* no break - we remove it right away */
   case PLUGIN_STATUS_DONE_AND_CLEANED:
   	{
@@ -815,6 +792,7 @@ struct arglist * plugins_scheduler_next(plugins_scheduler_t h)
     }
   l = l->next; 
   }
+
   
   /* Could not find anything */
   if((category == ACT_SCANNER ||
@@ -826,10 +804,13 @@ struct arglist * plugins_scheduler_next(plugins_scheduler_t h)
      }
      
    if(category + 1 >= ACT_DENIAL && flag && running_category < ACT_DENIAL)
+	{ 
    	return PLUG_RUNNING;  
+	}
  }
  
-   return flag == 0 ? NULL : PLUG_RUNNING; 
+
+ return flag != 0 ? PLUG_RUNNING : NULL;
 }
 
 
