@@ -1,13 +1,31 @@
 #ifndef PLUGINSCHEDULER_H
 #define PLUGINSCHEDULER_H
 
+
+
+struct scheduler_plugin {
+	int running_state;
+	int category;
+	int timeout;
+	struct arglist * required_ports;
+	struct arglist * required_udp_ports;
+	struct arglist * required_keys;
+	struct arglist * excluded_keys;
+	struct arglist * arglist;
+};
+
 #ifndef IN_SCHEDULER_CODE
 typedef void * plugins_scheduler_t;
 #else
+struct watch_list {
+	char * name;
+	struct watch_list * next;
+};
+
 
 struct hash {
 	char * name;
-	struct arglist * plugin;
+	struct scheduler_plugin * plugin;
 	char ** dependencies;
 	int num_deps;
 	char ** ports;
@@ -16,7 +34,7 @@ struct hash {
 
 struct list {
 	char * name;
-	struct arglist * plugin;
+	struct scheduler_plugin * plugin;
 	struct list * next;
 	struct list * prev;
 	};
@@ -39,7 +57,7 @@ typedef struct plugins_scheduler_struct * plugins_scheduler_t;
 #endif
 
 
-#define PLUG_RUNNING ((struct arglist*)0x02)
+#define PLUG_RUNNING ((struct scheduler_plugin*)0x02)
 #define PLUGIN_STATUS_UNRUN 		1
 #define PLUGIN_STATUS_RUNNING		2
 #define PLUGIN_STATUS_DONE		3
@@ -47,11 +65,11 @@ typedef struct plugins_scheduler_struct * plugins_scheduler_t;
 
 
 
-void plugin_set_running_state(struct arglist * , int);
+void plugin_set_running_state(plugins_scheduler_t, struct scheduler_plugin * , int);
 
 
-plugins_scheduler_t plugins_scheduler_init(struct arglist*, int);
-struct arglist * plugins_scheduler_next(plugins_scheduler_t);
+plugins_scheduler_t plugins_scheduler_init(struct arglist*, int, int);
+struct scheduler_plugin * plugins_scheduler_next(plugins_scheduler_t);
 
 void plugins_scheduler_free(plugins_scheduler_t);
 
