@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1998 - 2003 Renaud Deraison
+ * Copyright (C) 1998 - 2004 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2,
@@ -14,22 +14,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * In addition, as a special exception, Renaud Deraison
- * gives permission to link the code of this program with any
- * version of the OpenSSL library which is distributed under a
- * license identical to that listed in the included COPYING.OpenSSL
- * file, and distribute linked combinations including the two.
- * You must obey the GNU General Public License in all respects
- * for all of the code used other than OpenSSL.  If you modify
- * this file, you may extend this exception to your version of the
- * file, but you are not obligated to do so.  If you do not wish to
- * do so, delete this exception statement from your version.
  * 
  * Plugins requirements
  *
  */ 
  
 #include <includes.h>
+#include "pluginscheduler.h"
 #include "plugs_req.h"
 
 /**********************************************************
@@ -38,7 +29,7 @@
 	
 ***********************************************************/
  
-extern int kb_get_port_state_proto(struct arglist*, struct arglist*, int, char*);
+extern int kb_get_port_state_proto(struct kb_item **, struct arglist*, int, char*);
  
 /*---------------------------------------------------------
 
@@ -46,8 +37,8 @@ extern int kb_get_port_state_proto(struct arglist*, struct arglist*, int, char*)
  
  ----------------------------------------------------------*/
 static int
-get_closed_ports(keys, ports, preferences)
-   struct arglist * keys;
+get_closed_ports(kb, ports, preferences)
+   struct kb_item ** kb;
    struct arglist * ports;
    struct arglist * preferences;
 {
@@ -60,12 +51,13 @@ get_closed_ports(keys, ports, preferences)
    int iport = atoi(ports->name);			
    if(iport != 0)
    	{
-      	if( kb_get_port_state_proto(keys, preferences, iport, "tcp") != 0 )
+      	if( kb_get_port_state_proto(kb, preferences, iport, "tcp") != 0 )
 		return iport;
 	}
       else 
         {
-      	if( arg_get_value(keys, ports->name) != NULL )
+        
+      	if( kb_item_get_int(kb, ports->name) > 0 )
 		return 1; /* should be the actual value indeed ! */
 	}   
     ports = ports->next;
@@ -80,8 +72,8 @@ get_closed_ports(keys, ports, preferences)
  
  ------------------------------------------------------------*/
 static int
-get_closed_udp_ports(keys, ports, preferences)
-   struct arglist * keys;
+get_closed_udp_ports(kb, ports, preferences)
+   struct kb_item ** kb;
    struct arglist * ports;
    struct arglist * preferences;
 {   
@@ -90,7 +82,7 @@ get_closed_udp_ports(keys, ports, preferences)
   else while( ports->next != NULL)
   {
       int iport = atoi(ports->name);				
-      if(kb_get_port_state_proto(keys, preferences, iport, "udp"))return iport;
+      if(kb_get_port_state_proto(kb, preferences, iport, "udp"))return iport;
       ports = ports->next;
   }
   return 0; /* found nothing */
@@ -100,19 +92,20 @@ get_closed_udp_ports(keys, ports, preferences)
 /*-----------------------------------------------------------
             
 	     Returns the name of the first key
-	     which is not in <keyring>
+	     which is not in <kb>
 	    
  -----------------------------------------------------------*/
 static char * 
-key_missing(keyring, keys)
-  struct arglist * keyring;
+key_missing(kb, keys)
+  struct kb_item ** kb;
   struct arglist * keys;
 {
- if(!keyring || !keys)return NULL;
+ if(kb == NULL || keys == NULL )
+    return NULL;
  else {
    while( keys->next != NULL)
    {
-     if(arg_get_value(keyring, keys->name) == NULL)
+     if( kb_item_get_single(kb, keys->name, 0) == NULL )
       return keys->name;
      else
       keys = keys->next;
@@ -126,15 +119,16 @@ key_missing(keyring, keys)
 	    The opposite of the previous function
 	    
  -----------------------------------------------------------*/
-static char * key_present(keyring, keys)
- struct arglist * keyring;
+static char * key_present(kb, keys)
+ struct kb_item ** kb;
  struct arglist * keys;
 {
- if(!keyring || !keys)return NULL;
+ if( kb == NULL || keys == NULL )
+    return NULL;
  else {
    while( keys->next != NULL)
    {
-     if(arg_get_value(keyring, keys->name) != NULL)
+     if(kb_item_get_single(kb, keys->name, 0) != NULL)
       return keys->name;
      else
       keys = keys->next;
@@ -161,29 +155,27 @@ static char * key_present(keyring, keys)
  ------------------------------------------------------*/
 struct arglist * 
 requirements_common_ports(plugin1, plugin2)
- struct arglist * plugin1, *plugin2;
+ struct scheduler_plugin * plugin1, *plugin2;
 {
  struct arglist * ret = NULL;
- struct arglist * req1, *r1;
- struct arglist * req2, *r2;
+ struct arglist * req1;
+ struct arglist * req2;
  
  
  if(!plugin1 || !plugin2) return 0;
  
- r1 = req1 = plug_get_required_ports(plugin1);
- if(!req1)return 0;
- 
- r2 = req2 = plug_get_required_ports(plugin2);
- if(!req2)
- {
-  arg_free_all(r1);
-  return 0;
- }
+ req1 = plugin1->required_ports;
+ if ( req1 == NULL )
+	return 0;
+
+ req2 = plugin2->required_ports;
+ if ( req2 == NULL )
+	return 0;
  
  while(req1->next != NULL)
  {
   struct arglist * r = req2;
-  while(r && r->next)
+  if ( r != NULL  ) while( r->next != NULL )
   {
    if(req1->type == r->type)
    {
@@ -197,8 +189,6 @@ requirements_common_ports(plugin1, plugin2)
   }
   req1 = req1->next;
  }
- arg_free_all(r1);
- arg_free_all(r2);
  return ret;
 }
 
@@ -215,59 +205,58 @@ requirements_common_ports(plugin1, plugin2)
 
 char *
 requirements_plugin(kb, plugin, preferences)
- struct arglist * kb;
- struct arglist * plugin;
+ struct kb_item ** kb;
+ struct scheduler_plugin * plugin;
  struct arglist * preferences;
 {
   static char error[64];
   char * missing;
   char * present;
   struct arglist * tcp, * udp, * rkeys, * ekeys;
+  char	* opti = arg_get_value(preferences, "optimization_level");
+
   /*
    * Check wether the good ports are open
    */
-  tcp = plug_get_required_ports(plugin->value);
- 
-  
+  error[sizeof(error) - 1] = '\0';  
+  tcp = plugin->required_ports;
   if(tcp != NULL && (get_closed_ports(kb, tcp , preferences)) == 0)
      {
       strncpy(error, "none of the required tcp ports are open", sizeof(error) - 1);
-      arg_free_all(tcp);
       return error;
      }
-   if(tcp != NULL)arg_free_all(tcp);
       
-   udp = plug_get_required_udp_ports(plugin->value);  
+   udp = plugin->required_udp_ports;
    if(udp != NULL && (get_closed_udp_ports(kb, udp , preferences)) == 0)
       {
       strncpy(error, "none of the required udp ports are open", sizeof(error) - 1);
-      arg_free_all(udp);
       return error;
       }
-   if(udp != NULL)arg_free_all(udp);
-   
+
+   if (opti != NULL && (strcmp(opti, "open_ports") == 0 || atoi(opti) == 1))
+     return NULL;
+
   /*
    * Check wether a key we wanted is missing
    */
-  rkeys = plug_get_required_keys(plugin->value);
+  rkeys = plugin->required_keys;
   if((missing = key_missing(kb, rkeys)))
   {
      snprintf(error,sizeof(error), "because the key %s is missing", missing);
-     arg_free_all(rkeys);
      return error;
   }
-  if(rkeys != NULL)arg_free_all(rkeys);
   
+  if (opti != NULL && (strcmp(opti, "required_keys") == 0 || atoi(opti) == 2))
+     return NULL;
+
   /*
-   * Check wether a plugin we do not want is present
+   * Check wether a key we do not want is present
    */
-  ekeys = plug_get_excluded_keys(plugin->value);
+  ekeys = plugin->excluded_keys;
   if((present = key_present(kb, ekeys)))
   {
    snprintf(error,sizeof(error), "because the key %s is present", present);
-   arg_free_all(ekeys);
    return error;
   }
-  if(ekeys != NULL)arg_free_all(ekeys);
   return NULL;
 }
