@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1998 - 2001 Renaud Deraison
+ * Copyright (C) 1998 - 2004 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2,
@@ -14,25 +14,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * In addition, as a special exception, Renaud Deraison
- * gives permission to link the code of this program with any
- * version of the OpenSSL library which is distributed under a
- * license identical to that listed in the included COPYING.OpenSSL
- * file, and distribute linked combinations including the two.
- * You must obey the GNU General Public License in all respects
- * for all of the code used other than OpenSSL.  If you modify
- * this file, you may extend this exception to your version of the
- * file, but you are not obligated to do so.  If you do not wish to
- * do so, delete this exception statement from your version.
  *
  * Preferences  -- maps the content of the nessusd.conf file to memory
  *
  */
  
 #include <includes.h>
-#ifdef NESSUSNT
-#include "wstuff.h"
-#endif /* defined(NESSUSNT) */
 #include "comm.h"
 #include "preferences.h"
 #include "log.h"
@@ -69,7 +56,7 @@ int preferences_new(char * name)
   int f;
 
   if((f = open(name, O_CREAT | O_RDWR | O_EXCL, 0660))<0){
-    perror("prefrences_new():open ");
+    perror("preferences_new():open ");
     return(-1);
   }
 
@@ -86,6 +73,7 @@ int preferences_new(char * name)
  fprintf(fd, "# Niceness. If set to 'yes', nessusd will renice itself to 10.\n");
  fprintf(fd, "be_nice = no\n\n");
 
+
  fprintf(fd, "# Log file (or 'syslog') : \n");
  fprintf(fd, "logfile = %s\n\n", NESSUSD_MESSAGES);
  fprintf(fd, "# Shall we log every details of the attack ?\n");
@@ -120,7 +108,9 @@ int preferences_new(char * name)
  fprintf(fd, "\n\n# Safe checks rely on banner grabbing :\n");
  fprintf(fd, "safe_checks = yes\n");
  fprintf(fd, "\n\n# Automatically activate the plugins that are depended on\n");
- fprintf(fd, "auto_enable_dependencies = no\n");
+ fprintf(fd, "auto_enable_dependencies = yes\n");
+ fprintf(fd, "\n\n# Do not echo data from plugins which have been automatically enabled\n");
+ fprintf(fd, "silent_dependencies = yes\n");
  fprintf(fd, "\n\n# Designate hosts by MAC address, not IP address (useful for DHCP networks)\n");
  fprintf(fd, "use_mac_addr = no\n");
  
@@ -152,9 +142,11 @@ int preferences_new(char * name)
  fprintf(fd, "# (10.0.0.1, then 10.0.0.2, 10.0.0.3 and so on..) but will attempt to\n");
  fprintf(fd, "# slice the workload throughout the whole network (ie: it will scan\n");
  fprintf(fd, "# 10.0.0.1, then 10.0.0.127, then 10.0.0.2, then 10.0.0.128 and so on...\n");
- fprintf(fd, "slice_network_addresses = no\n");
- fprintf(fd, "#end.\n");
+ fprintf(fd, "slice_network_addresses = no\n\n");
  
+ fprintf(fd, "# Should consider all the NASL scripts as being signed ? (unsafe if set to 'yes')\n");
+ fprintf(fd, "nasl_no_signature_check = no\n\n");
+ fprintf(fd, "#end.\n");
  
   fclose(fd);
   close(f);
@@ -282,9 +274,6 @@ int preferences_get_host_expansion(preferences)
  if(strstr(pref, "ip"))ret = ret |  HG_SUBNET;
  }
  
- pref = arg_get_value(preferences, "ping_hosts");
- if(pref && strstr(pref, "yes"))ret = ret | HG_PING;
- 
  pref = arg_get_value(preferences, "reverse_lookup");
  if(pref && strstr(pref, "yes"))ret = ret | HG_REVLOOKUP;
  return ret;
@@ -717,6 +706,35 @@ preferences_upload_suffixes(preferences, fname)
  efree(&delme);
  return 0;
 }
+
+
+int
+preferences_nasl_no_signature_check(preferences)
+ struct arglist * preferences;
+{
+ static int yes = -1;
+ char * pref;
+ 
+ if(!preferences)
+  {
+   yes = -1;
+   return -1;
+  }
+  
+  
+ if(yes >= 0)
+  return yes;
+  
+  
+ pref = arg_get_value(preferences, "nasl_no_signature_check");
+ if(pref && !strcmp(pref, "yes"))
+   yes = 1;
+ else
+   yes = 0;
+
+ return yes;
+}
+
 int
 preferences_detached_scan(preferences)
  struct arglist * preferences;
@@ -878,6 +896,32 @@ preferences_detached_scan_email(preferences)
 }
 
 
+int
+preferences_silent_dependencies(preferences)
+ struct arglist * preferences;
+{
+ static int yes = -1;
+ char * pref;
+ 
+ if(!preferences)
+  {
+   yes = -1;
+   return -1;
+  }
+  
+  
+ if(yes >= 0)
+  return yes;
+  
+  
+ pref = arg_get_value(preferences, "silent_dependencies");
+ if(pref && !strcmp(pref, "yes"))
+   yes = 1;
+ else
+   yes = 0;
+
+ return yes;
+}
 
 
 #ifdef NESSUS_ON_SSL
@@ -919,4 +963,5 @@ preferences_reset_cache()
  preferences_delay_between_scans(NULL);
  preferences_detached_scan(NULL);
  preferences_detached_scan_email(NULL);
+ preferences_silent_dependencies(NULL);
 }
